/*
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
12V PWM Lighting Controller
rev 1.01 1/20/2013
Tested with Arduino 1.x environment.
License: Public Domain, share and enjoy
Code by John Norman, ACCX Products
jnorman@accxproducts.com


Description:
This program enables control of up to 4 channels of LED/other lighting from the "4-channel Lighting Controller" board rev 1.0
It uses a "RBBB" style Arduino with Atmega168 CPU, and a TI LMD18400N high-side driver with 4x3A outputs. All outputs may be
PWM dimmed/controller. This project is perfect for controlling 5m LED rope lighting with common cathode wiring.

Board with everything on it is available from:

https://www.batchpcb.com/pcbs/103377

Eagle files and latest version of the code:
http://www.accxproducts.com/wiki/index.php?title=Miscellaneous_Projects

Logic works as follows:

1. Disconnect the tailgate/door switch (SW2) from ground turns on R,G,B (White) lights
2. Pushing the button attached to (SW1) cycles through all lighting modes. This is for the camper/occupied area.
3. Pushing the button attached to (SW3) cycles through White or Red modes. This is for activating from the console/truck cab.
4. The lights turn off after TIMEOUT value. Default is 60 minutes.

/*


PD4 = D4 Power output enable
PD5 = D5 Output1  (PWM)
PD6 = D6 Output2  (PWM)
PB1 = D9 Output3  (PWM)
PB2 = D10 Output4 (PWM)

PD2 = D2 Input1   (INT)
PD3 = D3 Input2   (INT)
PC0 = A0/D14 Input3
PC1 = A1/D15 Input4
*/

#define ENABLE 4
#define RED    5
#define GREEN  9
#define BLUE   6
#define AUX    10

#define SW1    2
#define SW2    3
#define SW3    14
#define SW4    15

#define TIMEOUT 3600000 // Timeout value for turning off lights 60,000 = 1 minute


int  currentProgram = 0;
long timerValue = 0;
boolean doorOpened = 0;

void setup() {                
  // Initialize the I/O pins
  
  pinMode(ENABLE, OUTPUT);     
  pinMode(RED,    OUTPUT);  
  pinMode(GREEN,  OUTPUT);  
  pinMode(BLUE,   OUTPUT);  
  
  pinMode(SW1,    INPUT);  
  pinMode(SW2,    INPUT); 
  pinMode(SW3,    INPUT); 
  pinMode(SW4,    INPUT);   
  
  // Set power output to be initially disabled.
  digitalWrite(ENABLE, LOW);



Serial.begin(9600);
}


void loop() 
{


  
//inputTest(100);  // Test routines
//colorTest(100);  

checkSwitch1();    // Check if SW1 (bed switches) pressed and change program
checkSwitch3();    // Check if SW2 (console switch) pressed and change program

if(checkSwitch2() == 1 && doorOpened == 0)    // Check if SW2 closed and start timer.
 {
       currentProgram = 1;
       doorOpened = 1;
       timerValue = millis();
 }
 
if(checkSwitch2() == 0 && doorOpened == 1)    
 {
   currentProgram = 0;
   doorOpened = 0;
   delay(250);
 }

if(millis()-timerValue >= TIMEOUT)  // Turn off lights after timeout value reached from opening.
 {
   currentProgram=0;
 }

switch(currentProgram) 

{
 case 0:
  {

    digitalWrite(RED,   LOW);
    digitalWrite(GREEN, LOW);
    digitalWrite(BLUE,  LOW);    
    digitalWrite(ENABLE, LOW);
    break;
  }

 case 1:
  { 
    digitalWrite(ENABLE, HIGH);
    digitalWrite(RED,    HIGH);
    digitalWrite(GREEN,  HIGH);
    digitalWrite(BLUE,   HIGH);  
  
    
   break;
  }

   case 2:
  {
   digitalWrite(ENABLE, HIGH);
   analogWrite(RED,    255);
   digitalWrite(GREEN,  LOW);
   digitalWrite(BLUE,   LOW);   
   break;
  }

  
   case 3:
  {
   digitalWrite(ENABLE, HIGH);
   analogWrite(RED,    150);
   analogWrite(GREEN,  255);
   analogWrite(BLUE,   0);   
   break;
  }
  
     case 4:
  {
   digitalWrite(ENABLE, HIGH);
   analogWrite(RED,    255);
   analogWrite(GREEN,  150);
   analogWrite(BLUE,   0);   
   break;
  }
  


   case 5:
  {
   digitalWrite(ENABLE, HIGH);
   analogWrite(GREEN,  32);  
   digitalWrite(RED,    LOW);
   digitalWrite(BLUE,   LOW);  

   break;
  }


  
  default:
  {
   digitalWrite(ENABLE, LOW);
   break;
  }
}
 

  
}  // End of loop()

void inputTest(int iterations)
{
int count=0;
 while(count < iterations) 
  {
    Serial.print("\n"); 
    Serial.println("Pass: "); 
    Serial.println(count); 
    Serial.print("Input 1:");                    // Digital input testing
    Serial.println(digitalRead(SW1));
    Serial.print("Input 2:");
    Serial.println(digitalRead(SW2));
    Serial.print("Input 3:");
    Serial.println(digitalRead(SW3));
    Serial.print("Input 4:");
    Serial.println(digitalRead(SW4));
    delay(1000);
  }
}

void colorTest(int iterations) 

{
 int count=0;
 
 while(count < iterations) 
  {
  digitalWrite(ENABLE, HIGH); // enable the outputs
  
  Serial.println("Color:RED");
  digitalWrite(RED, HIGH);    // turn the LED on (HIGH is the voltage level)
  delay(500);                // wait for a second
  digitalWrite(RED, LOW);     // turn the LED off
 
  Serial.println("Color:GREEN"); 
  digitalWrite(GREEN, HIGH);  // turn the LED on (HIGH is the voltage level)
  delay(500);                // wait for a second
  digitalWrite(GREEN, LOW);   // turn the LED off
  
  Serial.println("Color:BLUE");
  digitalWrite(BLUE, HIGH);   // turn the LED on (HIGH is the voltage level)
  delay(500);                // wait for a second
  digitalWrite(BLUE, LOW);    // turn the LED off
 
  delay(500);
  Serial.println("Color:WHITE");
  digitalWrite(RED, HIGH);    // turn the LED on (HIGH is the voltage level)
  digitalWrite(GREEN, HIGH);  // turn the LED on (HIGH is the voltage level)
  digitalWrite(BLUE, HIGH);   // turn the LED on (HIGH is the voltage level)
  delay(500);
  digitalWrite(RED, LOW);    // turn the LED on (HIGH is the voltage level)
  digitalWrite(GREEN, LOW);  // turn the LED on (HIGH is the voltage level)
  digitalWrite(BLUE, LOW);   // turn the LED on (HIGH is the voltage level)
 
  digitalWrite(ENABLE, LOW);  // disable the outputs
  count++;
  }
  
}

void checkSwitch1()          // Check for button presses
{
  while(digitalRead(SW1) == 0) 
   {

     if(currentProgram <4) 
      {
       currentProgram++;
       timerValue = millis();
      }
     else 
     {
       currentProgram =0;

     }
    delay(350);                                        // 250ms delay between calue changes
   }
   
}

boolean checkSwitch2()
{
  return digitalRead(SW2);
}



void checkSwitch3()          // Check for button presses
{
  while(digitalRead(SW3) == 1) 
   {

     if(currentProgram <2) 
      {

       currentProgram++;
       timerValue = millis();
      }
     else 
     {
       currentProgram=0 ;
     }
    delay(350);                                        // 250ms delay between calue changes
   }
   
}
